/* Article.cc: A pointer to an article inside a bucket. */

#include "threads.h"
#include "util.h"
#include "error.h"

#include "Article.h"
#include "String.h"
#include "Connection.h"
#include "Bucket.h"

extern "C" {
#include <stdio.h>
#include <string.h>
}

Article::Article(Bucket *bucket, FILE *file, long offset)
{
    fp = file;
    in_bucket = bucket;
    start_offset = offset;
    headers.add_headers(fp);
    body_offset = ftell(fp);
    body_read = 0;
}

Article::~Article()
{
    fclose(fp);
}

void Article::get_message_id(String *buf)
{
    const char *id;

    id = headers["Message-ID"];
    buf->append(id ? id : "<corrupt.article>");
}

/* Aieee.  Fix. */
void Article::write_head(Connection *conn)
{
    const char *s, *p;

    for (s = headers.first_header(); s; s = headers.next_header()) {
	p = strchr(s, '\n');
	while (p) {
	    conn->write(s, p - s);
	    conn->write("\r\n", 2);
	    s = p + 1;
	    p = strchr(s, '\n');
	}
	conn->write(s);
	conn->write("\r\n", 2);
    }
}

void Article::write_body(Connection *conn)
{
    String line;

    if (body_read)
	fseek(fp, body_offset, SEEK_SET);
    body_read = 1;
    while (read_line_clear(&line, fp) >= 0) {
	if (line.length() == 1 && *line.chars() == '.')
	    return;
	conn->write(&line);
	conn->write("\r\n", 2);
    }
    error("No article terminator found in bucket %l",
	  (long) in_bucket->number());
}

