/* Atomic_file.cc: Atomically updating a file. */

#include "threads.h"
#include "error.h"

#include "Atomic_file.h"
#include "String.h"

extern "C" {
#include <unistd.h>
#include <errno.h>
}

Atomic_file::Atomic_file(const char *filename)
{
    name.append(filename);
    old_name.append(&name);
    old_name.append(".old", 4);
    new_name.append(&name);
    new_name.append(".new", 4);
}

FILE *Atomic_file::open_read()
{
    FILE *fp;

    fp = fopen(name.chars(), "r");
    if (fp)
	return fp;
    fp = fopen(old_name.chars(), "r");
    if (fp) {
	if (rename(old_name.chars(), name.chars()) < 0) {
	    error("Can't restore %s to %s", old_name.chars(), name.chars());
	    fclose(fp);
	    return NULL;
	}
    }
    return fp;
}

FILE *Atomic_file::open_new()
{
    FILE *fp;

    if (unlink(new_name.chars()) < 0 && errno != ENOENT)
	warn("Can't unlink existing %s (%m), continuing.", new_name.chars());

    /* Open the new file. */
    fp = fopen(new_name.chars(), "w");
    if (!fp)
	error("Can't open %s: %m", new_name.chars());
    return fp;
}

int Atomic_file::update()
{
    unlink(old_name.chars());
    if (rename(name.chars(), old_name.chars()) < 0 && errno != ENOENT)
	error("Can't move %s aside: %m", name.chars());
    else if (rename(new_name.chars(), name.chars()) < 0)
	error("Can't move %s into place: %m", new_name.chars());
    else if (unlink(old_name.chars()) < 0 && errno != ENOENT)
	warn("Can't unlink %s (%m), continuing", old_name.chars());
    else
	return 0;
    return -1;
}

