/* Bucket.h: Interface to an article bucket file (thread-safe). */

#ifndef BUCKET_H
#define BUCKET_H

#include "Article.h"
#include "String.h"

extern "C" {
#include <stdio.h>
#include <sys/types.h>
}

class Bucket;
class Spool;
class Headers;

struct Article_list {
    int refs;
    Article_list *next;
    Article article;

    Article_list(Bucket *bucket, FILE *fp, long offset);
};

class Bucket : private Mutex {
    FILE *fp;
    String filename;
    Article_list *list;
    int part;
    time_t bucket_num;

  public:

    Bucket(FILE *file, const char *name, int partition, time_t number);
    ~Bucket();

    int partition() const { return part; }
    time_t number() const { return bucket_num; }
    Article *get_article(long offset);
    void release_article(Article *article);
    int find_header(long offset, const char *name, int len, String *buf);
    long write_article(Headers *headers, const char *body, int len);
    long current_offset() const;
    int consistency_check(Spool *spool, long offset);
};

#endif

