/* Config.h: Interface to the configuration file. */

#ifndef CONFIG_H
#define CONFIG_H

#include "String.h"
#include "Wildcard_list.h"

#define DEFAULT_CONFIG_FILE ETCDIR "/tns.conf"

struct Expire_spec {
    int min;
    int def;
    int max;

    int operator ==(const Expire_spec &spec);
};

class Config {

    /* Access controls. */
    Wildcard_list<bool> admin_access;
    Wildcard_list<bool> post_access;
    Wildcard_list<bool> read_access;
    Wildcard_list<bool> transfer_access;
    bool readpost;

    /* Expiration information */
    Wildcard_list<Expire_spec> expire_times;
    Expire_spec default_expire_time;
    int expire_incr;
    int expire_keep_for;
    int cancel_keep_for;

    /* Feeds. */
    bool feeds_bidirectional;
    char **feed_sites;
    Wildcard_list<bool> feed_groups;
    Wildcard_list<bool> feed_dists;
    Wildcard_list<bool> feed_paths;

    /* General information */
    String server_name;
    bool add_mime_headers;
    Wildcard_list<bool> dists;
    String organization;

    /* Moderators */
    Wildcard_list<char *> moderators;
    char *default_moderator;

    /* Control message access (newgroup) */
    Wildcard_list<bool> newgroup_allow;

    /* Path names */
    String path_newsgroups_file;
    String path_index_dir;
    String path_groups_dir;
    String path_pool_dir;
    String path_feeds_dir;

    char **schema;

    /* Internal methods. */
    void read_config_file(const char *filename);
    void section_access(const char *s);
    void section_expire(const char *s);
    void section_feeds(const char *s);
    void section_general(const char *s);
    void section_moderators(const char *s);
    void section_newgroup(const char *s);
    void section_paths(const char *s);
    void section_schema(const char *s);
    time_t constrain_time(time_t t, time_t now, const Expire_spec *spec) const;

  public:
    Config(const char *config_filename);
    ~Config();

    void get_server_name(String *name) const;
    int any_distribution_accepted(const char *dist) const;
    bool get_add_mime_headers_flag() const;
    void get_default_organization(String *buf) const;
    void get_newsgroups_file(String *filename) const;
    void get_index_directory(String *dirname) const;
    void get_groups_directory(String *dirname) const;
    void get_pool_directory(String *dirname) const;
    void get_feeds_directory(String *dirname) const;
    time_t decide_bucket(const char *newsgroups, time_t expires_header) const;
    int get_expire_keep_for() const;
    int get_cancel_keep_for() const;
    int find_in_schema(const char *name, int namelen) const;
    const char *const *get_schema() const;
    const char *const *get_feed_sites() const;
    Wildcard_list<bool> *get_feed_groups(const char *name) const;
    Wildcard_list<bool> *get_feed_dists(const char *name) const;
    Wildcard_list<bool> *get_feed_paths(const char *name) const;
    void get_moderator(const char *group, String *buf) const;
    bool get_newgroup_allow(const char *group, int grouplen,
			    const char *from) const;
    Wildcard_list<bool> *get_post_access(const char *name,
					 const char *ip) const;
    Wildcard_list<bool> *get_read_access(const char *name,
					 const char *ip) const;
    Wildcard_list<bool> *get_transfer_access(const char *name,
					     const char *ip) const;
    bool get_admin_access(const char *name, const char *ip) const;
};

#endif

