/* Feeds.cc: The newsfeeds manager */

#include "threads.h"
#include "util.h"

#include "Feeds.h"
#include "Newsfeed.h"
#include "String.h"
#include "Config.h"

Feeds::Feeds(Spool *spool, const Config *config)
{
    String dirname;
    const char *const *sites;
    Newsfeed_list *entry;

    /* Get the feeds directory name, try to ensure that it exists. */
    config->get_feeds_directory(&dirname);
    make_directories_in_path(dirname.chars());

    /* Create a newsfeed for each specification in the configuration file. */
    newsfeeds = NULL;
    for (sites = config->get_feed_sites(); *sites; sites++) {
	entry = new Newsfeed_list();
	entry->feed = new Newsfeed(*sites, spool, config);
	entry->next = newsfeeds;
	newsfeeds = entry;
    }
}

Feeds::~Feeds()
{
    Newsfeed_list *next;

    while (newsfeeds) {
	next = newsfeeds->next;
	delete newsfeeds->feed;
	delete newsfeeds;
	newsfeeds = next;
    }
}

int Feeds::propagate_article(int partition, time_t bucket_num, long offset,
			     const Headers *headers, const char *body)
{
    Newsfeed_list *entry;

    for (entry = newsfeeds; entry; entry = entry->next) {
	if (entry->feed->propagate_article(partition, bucket_num, offset,
					   headers, body) < 0)
	    return -1;
    }

    return 0;
}

