/* Groups.h: Interface to the hierarchy of group files (thread-safe). */

#ifndef GROUPS_H
#define GROUPS_H

#include "Newsgroup.h"
#include "String.h"

extern "C" {
#include <stdio.h>
#include <sys/types.h>
}

class Atomic_file;
class Config;
class Connection;
class Headers;
template<class Value> class Wildcard_list;

struct Newsgroup_list : public Mutex {
    char *name;
    int first;
    int last;
    int num;
    int first_unreserved;
    int refs;
    time_t creation;
    char *flags;
    char *desc;
    bool in_file;
    Newsgroup *newsgroup;
    Newsgroup_list *next;
};

class Groups : private Reader_writer_lock {
    Mutex update_lock;
    const Config *config;
    String newsgroups_filename;
    String dirname;
    Atomic_file *newsgroups_file;
    Atomic_file *bounds_file;
    Newsgroup_list **hashtab;
    int num_groups;
    int size;
    time_t newsgroups_up_to_date;

    void read_newsgroups_file(FILE *fp);
    void read_bounds_file();
    int add_group_hashtable_entry(const char *name, int len,
				  const char *flags, int flagslen,
				  const char *desc, time_t creation);
    void expand_hash_table();
    Newsgroup_list *find_table_entry(const char *name, int len);
    Newsgroup_list *follow_aliases(const char *name);
    void construct_group_name_clear(String *buf, const char *name);
    void update_newsgroups_file();

  public:
    Groups(const Config *config_arg);
    ~Groups();

    Newsgroup *find_group(const char *name);
    void release_group(Newsgroup *group);
    void reserve_positions(const char *groups,
			   const Wildcard_list<bool> *access, String *xref);
    int add_to_groups(const Headers *headers, int num_lines, int len,
		      int partition, time_t bucket_num, long offset,
		      int check);
    int write_new_bounds_file();
    void send_grouplist(Connection *conn, const Wildcard_list<bool> *access);
    void send_group_descriptions(Connection *conn,
				 const Wildcard_list<bool> *access);
    void send_new_groups(Connection *conn, time_t mintime, const char *dist,
			 const Wildcard_list<bool> *access);
    int eliminate_bucket(time_t bucket_num);
    int verify_post_access(const char *groups,
			   const Wildcard_list<bool> *access, String *which,
			   String *mod);
    int any_groups_moderated(const char *groups);
    void add_group(const char *group, int grouplen,
		   const char *flags, int flagslen,
		   const char *desc, int desclen, time_t creation);
    void merge_groups(FILE *fp);
    int cancel_article(const char *xref);
    void check_newsgroups_file();
};

#endif

