/* Index.h: Interface to the message-ID index. */

#ifndef INDEX_H
#define INDEX_H

#include "String.h"

extern "C" {
#include <sys/types.h>
}

class Config;
class Dbz;

class Index : private Mutex {
    const Config *config;
    String index_name;
    int flatfile_writer;
    int flatfile_rover;
    long position;
    Dbz *db;
    int expire_keep_for;
    int cancel_keep_for;

    int get_line(const char *id, char *buf, int bufsize, long *offset);
    int store_line(const char *id, const char *line, int len);
    long find_entry(const char *id);

  public:
    Index(const Config *config_arg);
    ~Index();

    int lookup(const char *id, int *partition, time_t *bucket_num,
	       long *offset);
    int present(const char *id);
    int store(const char *id, int partition, time_t bucket_num, long offset);
    int store_dead_entry(const char *id);
    int overwrite_with_dead_entry(const char *id);
    int sync();
    int eliminate_bucket(time_t bucket_num);
};

#endif

