/* Newsgroup.h: Reader interface to newsgroup files (thread-safe). */

#ifndef NEWSGROUP_H
#define NEWSGROUP_H

#include "util.h"

#include "String.h"

extern "C" {
#include <stdio.h>
#include <sys/types.h>
}

class Connection;
class Config;
class Spool;

struct Index_entry {
    int index;
    int partition;
    time_t bucket_num;
    long offset;
    char *overdata;
};

class Newsgroup : private Mutex {
    const Config *config;
    String group_name;
    Index_entry *entries;
    int num_entries;
    int entries_size;
    int last;

    void add_index_entry(int index, int partition, time_t bucket_number,
			 long offset, const char *overdata);
    int find_entry(int index);

  public:
    Newsgroup(const char *name, FILE *fp, const Config *config_arg,
	      int last_arg);
    ~Newsgroup();

    const char *name() { return group_name.chars(); }

    int num_articles();
    int first_index();
    int last_index();

    int lookup(int index, int *partition, time_t *bucket_num, long *offset);
    int next(int *index, int *partition, time_t *bucket_num, long *offset);
    int prev(int *index, int *partition, time_t *bucket_num, long *offset);
    int first(int *index, int *partition, time_t *bucket_num, long *offset);

    void add_entry(int index, int partition, time_t bucket_num, long offset,
		   const char *overdata);
    void eliminate_bucket(time_t bucket_num);
    void eliminate_index(int ind);

    void send_overview(Connection *conn, int low, int high);
    void send_headers(Connection *conn, const char *name, int namelen,
		      int low, int high, Spool *spool);
};

#endif

