/* Pool.h: Interface to the pool of article buckets (thread-safe). */

#ifndef Pool_H
#define Pool_H

#include "Bucket.h"
#include "Array.h"

extern "C" {
#include <sys/types.h>
}

class Atomic_file;
class Config;
class Spool;

struct Bucket_list {
    int refs;
    Bucket_list *next;
    long consistent_at;
    Bucket bucket;

    Bucket_list(FILE *fp, const char *name,
		int partition, time_t bucket_number);
};

struct Partition {
    Bucket_list *buckets;
    int space;
};

typedef Array<struct Partition, 4, 4> Partitions;

class Pool : private Mutex {
    Spool *spool;
    String dirname;
    Atomic_file *checkpoint;
    Partitions partitions;
    int npart;

    void release_list_entry(Bucket_list **entry_ptr);
    void open_and_add_bucket(int partition, const char *name,
			     time_t bucket_number);
    Bucket_list *search_buckets(int partition, time_t bucket_number);

  public:
    /* Constructor and destructor. */
    Pool(Spool *spool_arg, const Config *config);
    ~Pool();

    Bucket *find_bucket(int partition, time_t bucket_number);
    Bucket *find_or_create_bucket(int partition, time_t bucket_number);
    void release_bucket(Bucket *bucket);
    int decide_partition(int num_bytes);
    int write_checkpoint_record();
    int consistency_check();
    time_t first_bucket();
    void expire_bucket(time_t bucket_num);
    int mark_cancelled(int partition, time_t bucket_num, long offset);
};

#endif

