/* String.c: Implementation of the String class. */

#include "threads.h"
#include "String.h"

#include "util.h"

extern "C" {
#include <ctype.h>
#include <string.h>
}

const int DELTA = 32;
const int MIN_POWER = 7;

void String::initialize(int anticipated_length)
{
    string_length = 0;
    string_size = adjust_size(anticipated_length + 1, MIN_POWER, DELTA);
    string_chars = new char[string_size];
    *string_chars = 0;
}

String::String()
{
    initialize(0);
}

String::String(int anticipated_length)
{
    initialize(anticipated_length);
}

String::String(const char *s)
{
    int len = strlen(s);

    initialize(strlen(s));
    string_length = len;
    memcpy(string_chars, s, len + 1);
}

String::String(const char *s, int len)
{
    initialize(len);
    string_length = len;
    memcpy(string_chars, s, len);
    string_chars[len] = 0;
}

String::String(const String &str)
{
    initialize(str.length());
    string_length = str.length();
    memcpy(string_chars, str.chars(), string_length + 1);
}

String::~String()
{
    delete[] string_chars;
}

void String::append(const char *s, int len)
{
    char *old;

    if (string_size < string_length + len + 1) {
	old = string_chars;
	string_size = adjust_size(string_length + len + 1, MIN_POWER, DELTA);
	string_chars = new char[string_size];
	memcpy(string_chars, old, string_length);
	delete[] old;
    }
    memcpy(string_chars + string_length, s, len);
    string_length += len;
    string_chars[string_length] = 0;
}

void String::truncate(int len)
{
    string_length = len;
    string_chars[len] = 0;
}

void String::shift(int start)
{
    memmove(string_chars, string_chars + start, string_length - start);
    truncate(string_length - start);
}

void String::reverse()
{
    reverse_chars(string_chars, string_length);
}

void String::strip_trailing_whitespace()
{
    char *p;

    p = string_chars + string_length - 1;
    while (p >= string_chars && isspace(*p))
	p--;
    truncate(p + 1 - string_chars);
}

void String::substitute(const char *what, const char *with)
{
    char *p;

    p = strpbrk(string_chars, what);
    while (p) {
	*p = with[strchr(what, *p) - what];
	p = strpbrk(p + 1, what);
    }
}

