/* Wildcard_list.h: Declarations for matching a list of wildcard patterns. */

#pragma interface

#ifndef WILDCARD_LIST_H
#define WILDCARD_LIST_H

#include "String.h"

template<class Value> struct Wildcard_entry;

template<class Value> class Wildcard_list {
    Wildcard_entry<Value> *entries;
    int num;
    int size;
    void (*destructor)(Value value);

  public:
    Wildcard_list(void (*destructor_arg)(Value value) = NULL);
    ~Wildcard_list();
    void add_pattern(const char *const *patterns, Value value);
    void add_pattern(const char *pattern, Value value);
    Value match(const char *const *strings, Value default_val) const;
    Value match(const char *s, Value default_val) const;
    int num_entries() const;
    int is_degenerate(Value default_value, Value *always_is) const;

    /* Constructs a sublist of the patterns for which prefix is a valid
     * partial match.  The lifetime of the sublist must be shorter than
     * the lifetime of the parent list. */
    Wildcard_list<Value> *sublist(const char *const *strings) const;
};

#endif

