/* error.cc: Deal with error conditions. */

#include "threads.h"
#include "error.h"

extern "C" {
#include <stdio.h>
#include <stdlib.h>
#include <stdarg.h>
#include <string.h>
#include <errno.h>
#include "syslog.h"
}

void error_init(char *zero_arg)
{
    char *path;

    if (zero_arg) {
	path = strrchr(zero_arg, '/');
	path = (path) ? path + 1 : zero_arg;
    } else {
	path = "tns";
    }
    openlog(path, LOG_NOWAIT, LOG_NEWS);
}

void crash(char *fmt, ...)
{
    va_list args;

    va_start(args, fmt);
    vsyslog(LOG_CRIT, fmt, args);
    va_end(args);
    abort();
}

void die(char *fmt, ...)
{
    va_list args;

    va_start(args, fmt);
    vsyslog(LOG_CRIT, fmt, args);
    va_end(args);
    exit(1);
}

void error(char *fmt, ...)
{
    va_list args;

    va_start(args, fmt);
    vsyslog(LOG_ERR, fmt, args);
    va_end(args);
}

void warn(char *fmt, ...)
{
    va_list args;

    va_start(args, fmt);
    vsyslog(LOG_WARNING, fmt, args);
    va_end(args);
}

void log(char *fmt, ...)
{
    va_list args;

    va_start(args, fmt);
    vsyslog(LOG_INFO, fmt, args);
    va_end(args);
}

