/* threads.cc: Thread-related functions. */

#include "threads.h"
#include "platform.h"

extern int debug;

void start_thread(void *(*start_routine)(void *), void *arg)
{
    pthread_attr_t attr;
    pthread_t thread;

    pthread_attr_init(&attr);
    pthread_attr_setdetachstate(&attr, PTHREAD_CREATE_DETACHED);
    if (debug)
	pthread_attr_setschedpolicy(&attr, SCHED_FIFO);
    pthread_create(&thread, &attr, start_routine, arg);
    pthread_attr_destroy(&attr);
};

Reader_writer_lock::Reader_writer_lock()
{
    num_readers = 0;
}

void Reader_writer_lock::read_lock()
{
    lock.lock();
    num_readers++;
    lock.unlock();
}

void Reader_writer_lock::read_done()
{
    int none;

    lock.lock();
    num_readers--;
    none = (num_readers == 0);
    lock.unlock();
    if (none)
	no_readers.signal();
}

void Reader_writer_lock::write_lock()
{
    /* Lock and wait for there to be no readers, and leave with the lock. */
    lock.lock();
    while (num_readers > 0)
	no_readers.wait(&lock);
}

void Reader_writer_lock::write_done()
{
    lock.unlock();
    no_readers.signal();
}

void Quiesce_lock::read_lock()
{
    quiesce.lock();
    Reader_writer_lock::read_lock();
    quiesce.unlock();
}

void Quiesce_lock::write_lock()
{
    quiesce.lock();
    Reader_writer_lock::write_lock();
    quiesce.unlock();
}

