/* Article.cc: A pointer to an article inside a bucket. */

#include "threads.h"
#include "util.h"
#include "error.h"

#include "Article.h"
#include "String.h"
#include "Connection.h"
#include "Bucket.h"

extern "C" {
#include <stdio.h>
#include <string.h>
}

Article::Article(Bucket *bucket, FILE *file, long offset)
{
    fp = file;
    in_bucket = bucket;
    start_offset = offset;
    headers.add_headers(fp);
    body_offset = ftell(fp);
    body_read = 0;
}

Article::~Article()
{
    fclose(fp);
}

void Article::get_message_id(Stringbuf *buf)
{
    String id;

    id = headers["Message-ID"];
    buf->append(id.valid() ? id : String("<corrupt.article>"));
}

void Article::write_head(Connection *conn)
{
    String s;
    int pos;

    for (s = headers.first_header(); s.valid(); s = headers.next_header()) {
	while ((pos = s.find('\n')) != -1) {
	    conn->write(s.left(pos));
	    conn->write("\r\n");
	    s.shift(pos + 1);
	}
	conn->write(s);
	conn->write("\r\n");
    }
}

void Article::write_body(Connection *conn)
{
    Stringbuf line;

    if (body_read)
	fseek(fp, body_offset, SEEK_SET);
    body_read = 1;
    while (read_line_clear(&line, fp) >= 0) {
	if (line.len() == 1 && *line.str() == '.')
	    return;
	conn->write(line.str());
	conn->write("\r\n");
    }
    error("No article terminator found in bucket %d", in_bucket->number());
}

