/* Atomic_file.cc: Atomically updating a file. */

#include "threads.h"
#include "error.h"
#include "util.h"

#include "Atomic_file.h"
#include "String.h"
#include "Stringbuf.h"

extern "C" {
#include <unistd.h>
#include <errno.h>
}

Atomic_file::Atomic_file(String filename)
{
    Stringbuf buf;

    name.store(filename);
    buf.format_clear("%s.old", filename);
    old_name.store(buf.str());
    buf.format_clear("%s.new", filename);
    new_name.store(buf.str());
}

FILE *Atomic_file::open_read()
{
    return fopen(name.cstring(), "r");
}

FILE *Atomic_file::open_new()
{
    FILE *fp;

    if (unlink(new_name.cstring()) < 0 && errno != ENOENT)
	warn("Can't unlink existing %s (%m), continuing.", new_name.str());

    /* Open the new file. */
    fp = fopen(new_name.cstring(), "w");
    if (!fp)
	error("Can't open %s: %m", new_name.str());
    return fp;
}

int Atomic_file::update()
{
    if (unlink(old_name.cstring()) < 0 && errno != ENOENT)
	warn("Can't unlink existing %s (%m), continuing", old_name.str());
    if (link(name.cstring(), old_name.cstring()) < 0 && errno != ENOENT) {
	error("Can't move %s aside: %m", name.str());
    } else if (rename(new_name.cstring(), name.cstring()) < 0) {
	error("Can't move %s into place: %m", new_name.str());
    } else {
	if (unlink(old_name.cstring()) < 0 && errno != ENOENT)
	    warn("Can't unlink %s (%m), continuing", old_name.str());
	return 0;
    }
    return -1;
}

