/* Bucket.h: Interface to an article bucket file (thread-safe). */

#ifndef BUCKET_H
#define BUCKET_H

#include "Article.h"
#include "String.h"
#include "Spool.h"

extern "C" {
#include <stdio.h>
#include <sys/types.h>
}

class Bucket;
class Headers;

enum Bucket_type {
    BUCKET_NORMAL,		/* Normal bucket ('n') */
    BUCKET_TEMPORARY,		/* Temporary bucket ('t') */
    BUCKET_DRAINING		/* Bucket being drained ('d') */
};

struct Article_list {
    int refs;
    Article_list *next;
    Article article;

    Article_list(Bucket *bucket, FILE *fp, long offset);
};

class Bucket : public Mutex {
    Bucket_loc bloc;
    Bucket_type bucket_type;
    time_t expire_time;
    FILE *fp;
    Stored_string filename;
    Article_list *list;

    long find_last_article_offset();

  public:

    Bucket(FILE *file, String name, const Bucket_loc *bloc, Bucket_type type,
	   time_t expire_time);
    ~Bucket();

    /* The following may be called any time. */
    int partition() const { return bloc.partition; }
    int number() const { return bloc.bucket_num; }
    time_t expires() const { return expire_time; }
    Article *get_article(long offset);
    void release_article(Article *article);
    int find_header(long offset, String name, Stringbuf *buf);
    int mark_article_cancelled(long offset);
    void drain_begin();
    int drain_last_article(Stringbuf *article, long *offset);
    void drain_truncate(long offset);

    /* The following should only be called during initialization phase or
     * with transactions quiesced. */
    long current_offset() const;
    int scan_article(long offset, int *num_lines, int *num_bytes,
		     Headers *headers);

    /* The following should only be called with the pool or bucket locked. */
    Bucket_type type() const { return bucket_type; }
    long write_article(Headers *headers, String body);
};

#endif

