/* Config.h: Interface to the configuration file. */

#ifndef CONFIG_H
#define CONFIG_H

#include "Stringbuf.h"
#include "String.h"
#include "Wildcard_list.h"
#include "Array.h"

#define DEFAULT_CONFIG_FILE ETCDIR "/tns.conf"

struct Expire_spec {
    int min;
    int def;
    int max;

    int operator ==(const Expire_spec &spec);
};

class Config {

    /* Access controls. */
    Wildcard_decision_list admin_access;
    Wildcard_decision_list post_access;
    Wildcard_decision_list read_access;
    Wildcard_decision_list transfer_access;
    bool readpost;

    /* Expiration information */
    Wildcard_list<Expire_spec> expire_times;
    Expire_spec default_expire_time;
    int expire_incr;
    int expire_record_for;
    int cancel_record_for;

    /* Feeds. */
    bool feeds_bidirectional;
    Stringlist feed_sites;
    Wildcard_decision_list feed_groups;
    Wildcard_decision_list feed_dists;
    Wildcard_decision_list feed_paths;
    Wildcard_list<int> feed_buffer_sizes;

    /* General information */
    Stored_string server_name;
    bool add_mime_headers;
    Wildcard_decision_list dists;
    Stored_string organization;
    unsigned short nntp_port;
    unsigned short admin_port;

    /* Moderators */
    Wildcard_list<Stored_string> moderators;
    Stored_string default_moderator;

    /* Control message access (newgroup) */
    Wildcard_decision_list newgroup_allow;

    /* Path names */
    Stored_string path_sendmail;
    Stored_string path_newsgroups_file;
    Stored_string path_index_dir;
    Stored_string path_groups_dir;
    Stored_string path_pool_dir;
    Stored_string path_feeds_dir;

    Stringlist schema;

    /* Internal methods. */
    void read_config_file(String filename);
    void section_access(String s);
    void section_expire(String s);
    void section_feeds(String s);
    void section_general(String s);
    void section_moderators(String s);
    void section_newgroup(String s);
    void section_paths(String s);
    void section_schema(String s);
    time_t constrain_time(time_t t, time_t now, const Expire_spec *spec) const;

  public:
    explicit Config(String config_filename);
    ~Config();

    void get_server_name(Stringbuf *name) const;
    int any_distribution_accepted(String dist) const;
    bool get_add_mime_headers_flag() const;
    unsigned short get_nntp_port() const;
    unsigned short get_admin_port() const;
    void get_default_organization(Stringbuf *buf) const;
    void get_sendmail_file(Stringbuf *filename) const;
    void get_newsgroups_file(Stringbuf *filename) const;
    void get_index_directory(Stringbuf *dirname) const;
    void get_groups_directory(Stringbuf *dirname) const;
    void get_pool_directory(Stringbuf *dirname) const;
    void get_feeds_directory(Stringbuf *dirname) const;
    time_t decide_expire_time(String newsgroups, time_t expires_header) const;
    int get_expire_record_for() const;
    int get_cancel_record_for() const;
    int find_in_schema(String name) const;
    const Stringlist *get_schema() const;
    const Stringlist *get_feed_sites() const;
    Wildcard_decision_list *get_feed_groups(String name) const;
    Wildcard_decision_list *get_feed_dists(String name) const;
    Wildcard_decision_list *get_feed_paths(String name) const;
    int get_feed_buffer_size(String name) const;
    void get_moderator(String group, Stringbuf *buf) const;
    bool get_newgroup_allow(String group, String from) const;
    Wildcard_decision_list *get_post_access(String name, String ip) const;
    Wildcard_decision_list *get_read_access(String name, String ip) const;
    Wildcard_decision_list *get_transfer_access(String name, String ip) const;
    bool get_admin_access(String name, String ip) const;
};

#endif

