/* Feeds.cc: The newsfeeds manager */

#include "threads.h"
#include "util.h"

#include "Feeds.h"
#include "Newsfeed.h"
#include "Stringbuf.h"
#include "String.h"
#include "Config.h"

Feeds::Feeds(Spool *spool, const Config *config)
{
    Stringbuf dirname;
    const Stringlist *sites;
    Newsfeed_list *entry;
    int i;

    /* Get the feeds directory name, try to ensure that it exists. */
    config->get_feeds_directory(&dirname);
    make_directories_in_path(dirname.str());

    /* Create a newsfeed for each specification in the configuration file. */
    newsfeeds = NULL;
    sites = config->get_feed_sites();
    for (i = 0; i < sites->len(); i++) {
	entry = new Newsfeed_list();
	entry->feed = new Newsfeed((*sites)[i].str(), spool, config);
	entry->next = newsfeeds;
	newsfeeds = entry;
    }
}

Feeds::~Feeds()
{
    Newsfeed_list *next;

    while (newsfeeds) {
	next = newsfeeds->next;
	delete newsfeeds->feed;
	delete newsfeeds;
	newsfeeds = next;
    }
}

int Feeds::propagate_article(const Artloc *loc, const Headers *headers,
			     String body)
{
    Newsfeed_list *entry;

    for (entry = newsfeeds; entry; entry = entry->next) {
	if (entry->feed->propagate_article(loc, headers, body) < 0)
	    return -1;
    }

    return 0;
}

int Feeds::replace_article_references(const Artloc *, const Artloc *)
{
    /* Stub.  Drained articles are probably old and have been fed to all
     * sites, so we would probably go to a lot of work without making very
     * many changes.  One way of speeding this up would be to store a
     * translation table here (checked by Newsfeed::feed_article()) and
     * having each newsfeed rewrite its queue using the translation table
     * after it grows to a certain size (so we can flush it). */
    return 0;
}

int Feeds::sync_queues()
{
    Newsfeed_list *entry;

    for (entry = newsfeeds; entry; entry = entry->next) {
	if (entry->feed->sync_queue() < 0)
	    return -1;
    }

    return 0;
}

