/* Groups.h: Interface to the hierarchy of group files (thread-safe). */

#ifndef GROUPS_H
#define GROUPS_H

#include "Newsgroup.h"
#include "String.h"

extern "C" {
#include <stdio.h>
#include <sys/types.h>
}

class Atomic_file;
class Config;
class Connection;
class Headers;
class Wildcard_decision_list;
struct Artloc;

struct Newsgroup_list : public Mutex {
    Stored_string name;
    int first;
    int last;
    int num;
    int first_unreserved;
    int refs;
    time_t creation;
    Stored_string flags;
    Stored_string desc;
    bool in_file;
    Newsgroup *newsgroup;
    Newsgroup_list *next;
};

class Groups : private Reader_writer_lock {
    Mutex update_lock;
    const Config *config;
    Stored_string newsgroups_filename;
    Stored_string dirname;
    Atomic_file *newsgroups_file;
    Atomic_file *bounds_file;
    Newsgroup_list **hashtab;
    unsigned int num_groups;
    unsigned int size;
    time_t newsgroups_up_to_date;

    void read_newsgroups_file(FILE *fp);
    void read_bounds_file();
    int add_group_hashtable_entry(String name, String flags, String desc,
				  time_t creation);
    void expand_hash_table();
    Newsgroup_list *find_table_entry(String name);
    Newsgroup_list *follow_aliases(String name);
    void construct_group_name_clear(Stringbuf *buf, String name);
    void update_newsgroups_file();

  public:
    Groups(const Config *config_arg);
    ~Groups();

    Newsgroup *find_group(String name);
    void release_group(Newsgroup *group);
    void reserve_positions(String groups, const Wildcard_decision_list *access,
			   Stringbuf *xref);
    int add_to_groups(const Headers *headers, int num_lines, int len,
		      const Artloc *loc, int check);
    int write_new_bounds_file();
    void send_grouplist(Connection *conn,
			const Wildcard_decision_list *access);
    void send_group_descriptions(Connection *conn,
				 const Wildcard_decision_list *access);
    void send_new_groups(Connection *conn, time_t mintime, String dist,
			 const Wildcard_decision_list *access);
    int eliminate_bucket(const Bucket_loc *bloc);
    int verify_post_access(String groups, const Wildcard_decision_list *access,
			   Stringbuf *which, Stringbuf *mod);
    int any_groups_moderated(String groups);
    void add_group(String group, String flags, String desc, time_t creation);
    void merge_groups(FILE *fp);
    int cancel_article(String xref);
    void check_newsgroups_file();
};

#endif

