/* Index.h: Interface to the message-ID index. */

#ifndef INDEX_H
#define INDEX_H

#include "String.h"
#include "Spool.h"

extern "C" {
#include <sys/types.h>
}

class Config;
class Dbz;
struct Artloc;

class Index : private Mutex {
    const Config *config;
    Stored_string index_name;
    int flatfile_writer;
    int flatfile_rover;
    long position;
    Dbz *db;
    int expire_record_for; /* XXX */
    int cancel_record_for;

    int get_line(String id, char *buf, int bufsize, long *offset);
    int store_line(String id, String line);
    long find_entry(String id);

  public:
    Index(const Config *config_arg);
    ~Index();

    int lookup(String id, Artloc *loc);
    int present(String id);
    int store(String id, const Artloc *loc);
    int overwrite(String id, const Artloc *loc);
    int store_dead_entry(String id);
    int overwrite_with_dead_entry(String id);
    int sync();
    int eliminate_bucket(const Bucket_loc *bloc, time_t bucket_expire_time);
};

#endif

