/* Newsfeed.h: Interface to a site's newsfeed handler. */

#ifndef FEED_H
#define FEED_H

#include "String.h"

extern "C" {
#include <stdio.h>
#include <sys/types.h>
}

class Config;
class Headers;
class Connection;
class Spool;
struct Artloc;
template<class Value> class Wildcard_list;

class Newsfeed {
    Stored_string name;
    Spool *spool;
    Wildcard_list<bool> *groups_list;
    Wildcard_list<bool> *dists_list;
    Wildcard_list<bool> *paths_list;
    int work_flag;
    int file_flag;
    Condition work;
    Mutex queue_lock;
    Artloc *queue_buffer;
    int buffer_pos;
    int buffer_size;
    FILE *queue_file;
    Stored_string dirname;
    Stored_string queue_filename;
    Stored_string work_filename;

    int process_queue(Connection *conn);
    void queue_articles_from_file(FILE *fp);
    int feed_article(Connection *conn, const Artloc *loc);
    Connection *make_connection();
    int wanted(const Headers *headers);
    int queue_article(const Artloc *loc);
    int flush_buffer_to_queue_file();

  public:
    /* spec and dirname must be valid for the lifetime of the feed. */
    Newsfeed(String name_arg, Spool *spool_arg, const Config *config);
    ~Newsfeed();
    int propagate_article(const Artloc *loc, const Headers *headers,
			  String article);
    void feed_articles();
    int replace_article_references(const Artloc *oldloc, const Artloc *loc);
    int sync_queue();
};

#endif

