/* Newsgroup.h: Reader interface to newsgroup files (thread-safe). */

#ifndef NEWSGROUP_H
#define NEWSGROUP_H

#include "util.h"

#include "String.h"
#include "Spool.h"

extern "C" {
#include <stdio.h>
#include <sys/types.h>
}

class Connection;
class Config;
struct Artloc;

struct Article_entry {
    int number;
    Artloc loc;
    Stored_string overdata;
};

class Newsgroup : private Mutex {
    const Config *config;
    Stored_string group_name;
    Article_entry *entries;
    int num_entries;
    int entries_size;
    int last;

    void add_article_entry(int number, const Artloc *loc, String overdata);
    int find_entry(int number);

  public:
    Newsgroup(String name, FILE *fp, const Config *config_arg, int last_arg);
    ~Newsgroup();

    String name() { return group_name.str(); }

    int num_articles();
    int first_number();
    int last_number();

    int lookup(int number, Artloc *loc);
    int next(int *number, Artloc *loc);
    int prev(int *number, Artloc *loc);
    int first(int *number, Artloc *loc);

    void add_entry(int number, const Artloc *loc, String overdata);
    void eliminate_bucket(const Bucket_loc *bloc);
    void eliminate_article(int num);

    void send_overview(Connection *conn, int low, int high);
    void send_headers(Connection *conn, String name, int low, int high,
		      Spool *spool);
};

#endif

