/* Pool.h: Interface to the pool of article buckets (thread-safe). */

#ifndef Pool_H
#define Pool_H

#include "Bucket.h"
#include "Array.h"
#include "String.h"

extern "C" {
#include <sys/types.h>
}

class Atomic_file;
class Config;
class Spool;
class Connection;
struct Artloc;

struct Bucket_list {
    int refs;
    Bucket_list *next;
    long consistent_at;
    long checked_to;
    Bucket bucket;

    Bucket_list(FILE *fp, String name, const Bucket_loc *bloc,
		Bucket_type type, time_t expire_time);
};

struct Partition {
    Bucket_list *buckets;
    int space;
};

typedef Array<struct Partition, 4, 4> Partitions;

class Pool : private Mutex {
    Spool *spool;
    const Config *config;
    Stored_string dirname;
    Atomic_file *checkpoint;
    Partitions partitions;
    int npart;

    void release_list_entry(Bucket_list **entry_ptr);
    void open_and_add_bucket(const Bucket_loc *bloc, String name);
    Bucket_list *search_buckets(const Bucket_loc *bloc);
    Bucket_list *create_bucket(int partition, Bucket_type type,
			       time_t expire_time);

  public:
    /* Constructor and destructor. */
    Pool(Spool *spool, const Config *config);
    ~Pool();

    Bucket *find_bucket(const Bucket_loc *bloc);
    int store_article(Headers *headers, String body, time_t expire_time,
		      Artloc *loc, int partition = -1);
    void release_bucket(Bucket *bucket);
    int write_checkpoint_record();
    void begin_unfiled_articles();
    int next_unfiled_article(int *num_lines, int *num_bytes, Artloc *loc,
			     Headers *headers);
    int soonest_bucket_to_expire(Bucket_loc *bloc, time_t *expire_time);
    void expire_bucket(const Bucket_loc *bloc);
    int mark_cancelled(const Artloc *loc);
    void send_bucket_list(Connection *conn);
};

#endif

