/* String.h: Interface for read-only string objects. */

#ifndef STRING_H
#define STRING_H

extern "C" {
#include <sys/types.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
}

class String {
    const char *s;
    size_t l;

  public:
    /* Constructors.  Implicit type conversion from const char * is intended
     * to be used for string constants ONLY. */
    explicit String() { ; }
    explicit String(const char *chars, int len) { s = chars; l = len; }
    String(const char *chars) { s = chars; if (chars) l = strlen(chars); }

    /* Accessors. */
    bool valid() const { return (s != NULL); }
    int len() const { return l; }
    char operator *() const { return *s; }
    char operator [](int n) const { return s[n]; }
    void copy_into(char *dest) const { memcpy(dest, s, l); }
    ssize_t write(int fd) const;
    size_t fwrite(FILE *fp) const;

    /* Contents-parsing. */
    long tonum() const;
    unsigned long hash_case() const;
    int count_char(char c) const;
    int count_words() const;

    /* Searching. */
    int find(char c) const;
    int find(String str) const;
    int find_case(char c) const;
    int find_case(String str) const;
    int rfind(char c) const;
    int find_set(String str) const;

    /* Comparison. */
    friend int compare(String s1, String s2);
    friend int compare_case(String s1, String s2);
    int offset_from(String str) const { return s - str.s; }

    /* Mutators. */
    void shift(int n) { s += n; l -= n; }
    void trunc(int n) { l = n; }
    void narrow(int off, int len) { s += off; l = len; }
    void trunc_whitespace();
    void skip_whitespace();
    void strip_whitespace();
    String word();
    long number();

    /* Making strings from other strings. */
    String after(int n) const { return String(s + n, l - n); }
    String left(int n) const { return String(s, n); }
    String right(int n) const { return String(s + (l - n), n); }
    String substr(int off, int len) const { return String(s + off, len); }
    String get_words(String *result, int n, String *whitespace = NULL) const;
    String equivalent_substr(String old_backing, String new_backing) const;
};

class Stored_string {
    char *s;

  public:
    /* Constructors and destructors. */
    explicit Stored_string();
    explicit Stored_string(String str);
    Stored_string(const Stored_string &ss);
    ~Stored_string();

    /* Accessors. */
    String str() const { return String(s); }
    const char *cstring() const { return s; }
    int len() const { return strlen(s); }

    /* Mutators. */
    void store(String str);
};

#endif

