/* Stringbuf.h: Interface for a resizable string buffer. */

#ifndef STRINGBUF_H
#define STRINGBUF_H

#include "Array.h"
#include "String.h"

extern "C" {
#include <stdarg.h>
}

class Stringbuf {
    Array<char, 7, 32> buf;
    size_t l;

  public:
    /* Constructors. */
    explicit Stringbuf();
    explicit Stringbuf(size_t anticipated);
    explicit Stringbuf(String str);

    /* Accessors. */
    size_t len() const { return l; }
    String str() const { return String(buf.base(), l); }
    const char *cstring();

    /* Mutators. */
    void append(char c);
    void append(String str);
    void store(String str);
    void trunc(size_t len);
    void shift(size_t start);
    void clear();
    void reverse();
    void substitute(String set1, String set2, size_t start = 0);
    void format(String fmt, ...);
    void format_clear(String fmt, ...);
    void vformat(String fmt, va_list args);
};

#endif

