/* Transactions.h Interface to posting transactions list (thread-safe). */

#ifndef TRANSACTIONS_H
#define TRANSACTIONS_H

enum Post_status { POST_OK, POST_REJECTED, POST_FAILED };

struct Id_entry {
    String id;
    Post_status status;
    int collisions;
    Condition finished;
    Id_entry *next, **prev_ptr;
};

class Transactions : private Mutex {
    Id_entry *head;
    Quiesce_lock post_lock;

    int detect_collision(String id, Post_status *status);
    Id_entry *add_entry(String id);
    void release_entry(Id_entry *entry, Post_status status);

  public:
    Transactions();
    ~Transactions();

    Id_entry *beginning_post(String id, Post_status *status);
    void post_committed(Id_entry *entry);
    void post_failed(Id_entry *entry);
    Id_entry *beginning_cancel(String id);
    void cancel_done(Id_entry *entry);
    Id_entry *beginning_refile();
    void refile_done(Id_entry *entry);

    void quiesce();
    void quiesce_done();
};

#endif

