/* error.cc: Deal with error conditions. */

#include "threads.h"
#include "error.h"

#include "Stringbuf.h"

extern "C" {
#include <stdio.h>
#include <stdlib.h>
#include <stdarg.h>
#include <string.h>
#include <errno.h>
#include "syslog.h"
}

void error_init(String progname)
{
    int pos;
    Stringbuf *path = new Stringbuf;

    pos = progname.rfind('/');
    path->store((pos == -1) ? progname : progname.after(pos + 1));
    openlog(path->cstring(), LOG_NOWAIT, LOG_NEWS);
}

/* A programming error occurred.  Log at the critical level and dump core. */
void crash(String fmt, ...)
{
    va_list args;
    Stringbuf msg;

    va_start(args, fmt);
    msg.vformat(fmt, args);
    syslog(LOG_CRIT, "%s", msg.cstring());
    va_end(args);
    abort();
}

/* A fatal error condition occurred.  Log at the critical level and exit with
 * an error. */
void die(String fmt, ...)
{
    va_list args;
    Stringbuf msg;

    va_start(args, fmt);
    msg.vformat(fmt, args);
    syslog(LOG_CRIT, "%s", msg.cstring());
    va_end(args);
    exit(1);
}

/* Something happened which wasn't supposed to happen, and it's keeping us
 * from getting our work done.  Log at the error level. */
void error(String fmt, ...)
{
    va_list args;
    Stringbuf msg;

    va_start(args, fmt);
    msg.vformat(fmt, args);
    syslog(LOG_ERR, "%s", msg.cstring());
    va_end(args);
}

/* Something happened which wasn't supposed to happen, but we can deal with
 * it.  Log at the warning level. */
void warn(String fmt, ...)
{
    va_list args;
    Stringbuf msg;

    va_start(args, fmt);
    msg.vformat(fmt, args);
    syslog(LOG_WARNING, "%s", msg.cstring());
    va_end(args);
}

/* Something interesting happened. */
void log(String fmt, ...)
{
    va_list args;
    Stringbuf msg;

    va_start(args, fmt);
    msg.vformat(fmt, args);
    syslog(LOG_INFO, "%s", msg.cstring());
    va_end(args);
}

