/* threads.h: C++ wrapper for threads. */

#ifndef THREADS_H
#define THREADS_H

extern "C" {
#include <pthread.h>
#include <timers.h>
}

class Mutex {
friend class Condition;
    pthread_mutex_t mutex;

  public:
    Mutex() { pthread_mutex_init(&mutex, NULL); }
    ~Mutex() { pthread_mutex_destroy(&mutex); }
    void lock() { pthread_mutex_lock(&mutex); }
    void unlock() { pthread_mutex_unlock(&mutex); }
    void try_lock() { pthread_mutex_trylock(&mutex); }
};

class Condition {
    pthread_cond_t cond;

  public:
    Condition() { pthread_cond_init(&cond, NULL); }
    ~Condition() { pthread_cond_destroy(&cond); }
    void wait(Mutex *mutex) { pthread_cond_wait(&cond, &mutex->mutex); }
    int timedwait(Mutex *mutex, const struct timespec *abstime)
	{ return pthread_cond_timedwait(&cond, &mutex->mutex, abstime); }
    void signal() { pthread_cond_signal(&cond); }
    void broadcast() { pthread_cond_broadcast(&cond); }
};

class Reader_writer_lock {
    Mutex lock;
    Condition no_readers;
    int num_readers;

  public:
    Reader_writer_lock();
    void read_lock();
    void read_done();
    void write_lock();
    void write_done();
};

class Quiesce_lock : public Reader_writer_lock {
    Mutex quiesce;

  public:
    void read_lock();
    void write_lock();
};

void start_thread(void *(*start_routine)(void *), void *arg);

#endif

