/* util.h: Declarations for miscellaneous utilities. */

#ifndef UTIL_H
#define UTIL_H

#include "threads.h"
#include "String.h"

const int NUMBER_BUF_SIZE = 32;
typedef char Number_buf[NUMBER_BUF_SIZE];

class Stringbuf;

extern "C" {
#include <stdio.h>
#include <stdarg.h>
#include <sys/types.h>
}

/* Number of characters to use to represent a numbers using %N in
 * format().  Make sure zero_filler[] in format() is as long as
 * this number. */
#define NNTP_NUMBER_LEN 10

int wildcard_match(String pattern, String s);

/* Formatting operations. */
String long_to_ascii(long num, Number_buf nbuf, int width);
String long_to_radix32(unsigned long num, Number_buf nbuf);

/* Reading lines from files. */
int read_line(Stringbuf *line, FILE *fp);
int read_line_clear(Stringbuf *line, FILE *fp);

/* System services.  Ports are in network byte order. */
int tcp_connect(String name, String service, unsigned short fallback,
		Stringbuf *error);
unsigned short service_port(String service, unsigned short fallback);
int blocksize(int fd);

/* Specialized routines for NNTP, the spool, etc. */
int message_id_equal(String id1, String id2);
int verify_message_id(String id);
time_t parse_header_date(String date);
time_t parse_nntp_date(String date, String timest, String nntp);
int generate_current_header_date(Stringbuf *buf);
String get_delimited_element(String *str, char delimiter);
String get_group_name(String *groups);
void make_directories_in_path(String name);
int check_flags(String flags);

class Counter : private Mutex {
    int n;

  public:
    Counter() { n = 0; }
    Counter(int start) { n = start; }
    int next() { int r; lock(); r = n++; unlock(); return r; }
};

#endif

