/* Headers_test.cc: Test driver for Headers data type. */

#include "threads.h"
#include "util.h"

#include "Headers.h"
#include "String.h"

extern "C" {
#include <stdio.h>
#include <string.h>
}

int main()
{
    Headers headers;
    FILE *fp;
    const char *p;
    String str;

    util_init();
    /* Test reading from a file. */
    fp = fopen("header1.input", "r");
    if (fp) {
	if (headers.add_headers(fp) >= 0) {
	    p = headers.find_value("path");
	    if (!p || strncmp(p, "senator-bed", 11) != 0)
		fprintf(stderr, "Bad path value: %s\n", p);
	    p = headers.find_value("fRom");
	    if (!p || strcmp(p, "rpwhite@shell.portal.com (rpwhite)") != 0)
		fprintf(stderr, "Bad from value: %s\n", p);
	    p = headers.find_value("Organization");
	    if (!p || strncmp(p, "Portal\n  ", 9) != 0)
		fprintf(stderr, "Bad organization value: %s\n", p);
	    p = headers.find_value("X-Testbar", 6);
	    if (!p || strcmp(p, "Tab\n\tContinuations") != 0)
		fprintf(stderr, "Bad X-Test value: %s\n", p);
	    if (headers.find_value("foobar"))
		fprintf(stderr, "Found erroneous value for foobar.\n");
	} else {
	    fprintf(stderr, "Headers failed to parse header1.input.\n");
	}
	if (read_line(&str, fp) == 0) {
	    if (strncmp(str.chars(), "Despite 19 years", 16) != 0)
		fprintf(stderr, "Read wrong body line: %s\n", str.chars());
	} else {
	    fprintf(stderr, "At end of file after headers.\n");
	}
    } else {
	fprintf(stderr, "Can't open header1.input for reading.\n");
    }

    /* Test clearing and reading from a string. */
    headers.clear();
    p = headers.add_headers("Path: pathline\na:1\n 2\nb : 3\n\t4\n\nfee");
    if (p) {
	if (strcmp(p, "fee") != 0)
	    fprintf(stderr, "Wrong position after headers: %s\n", p);
	p = headers.find_value("Path");
	if (!p || strcmp(p, "pathline") != 0)
	    fprintf(stderr, "Bad path value: %s\n", p);
	p = headers.find_value("a");
	if (!p || strcmp(p, "1\n 2") != 0)
	    fprintf(stderr, "Bad a value: %s\n", p);
	p = headers.find_value("b");
	if (!p || strcmp(p, "3\n\t4") != 0)
	    fprintf(stderr, "Bad b value: %s\n", p);
	if (headers.find_value("Organization"))
	    fprintf(stderr, "Found erroneous valuef for Organization.\n");

	/* Test traversal. */
 	p = headers.first_header();
	if (!p || strcmp(p, "Path: pathline") != 0)
	    fprintf(stderr, "Wrong first header: %s.\n", p);
 	p = headers.next_header();
	if (!p || strcmp(p, "a:1\n 2") != 0)
	    fprintf(stderr, "Wrong second header: %s.\n", p);
 	p = headers.next_header();
	if (!p || strcmp(p, "b : 3\n\t4") != 0)
	    fprintf(stderr, "Wrong third header: %s.\n", p);
	p = headers.next_header();
	if (p)
	    fprintf(stderr, "Erroneous fourth headers: %s\n", p);
    }

    return 0;
}

