/* String_test.cc: Unit test driver for string type. */

#include "threads.h"
#include "util.h"

#include "String.h"

extern "C" {
#include <stdio.h>
#include <string.h>
}

static void verify(String *str, char *value);
static int err = 0;

int main()
{
    String str;

    util_init();
    verify(&str, "");
    str.append("breadth", 3);
    verify(&str, "bre");
    str.append("tron");
    verify(&str, "bretron");
    str.append('p');
    verify(&str, "bretronp");
    str.strip_trailing_whitespace();
    verify(&str, "bretronp");
    str.append("   ");
    verify(&str, "bretronp   ");
    str.strip_trailing_whitespace();
    verify(&str, "bretronp");
    str.truncate(4);
    verify(&str, "bret");
    str.shift(2);
    verify(&str, "et");
    str.append("foobar  ");
    verify(&str, "etfoobar  ");
    str.reverse();
    verify(&str, "  raboofte");
    String str2("record");
    str2.append(&str);
    verify(&str2, "record  raboofte");
    return err;
}

static void verify(String *str, char *value)
{
    if (strcmp(str->chars(), value) != 0)
	fprintf(stderr, "Value mismatch: %s vs %s\n", str->chars(), value);
    if (str->length() != strlen(value))
	fprintf(stderr, "Length mismatch: %d vs %s\n", str->length(), value);
    err = 1;
}

